/*
 *	pa.h -- Definitions for Parsonal Arm PA10
 *
 *	Copyright (c) Mitsubisgi Heavy Industries, 1993
 *
 */

#ifndef _PA_H_
#define _PA_H_

#define	_PA_	10

/* 
 * 	data types with specific significance :
 */
typedef float 	MATRIX[3][4];	/* 3x4 matrix for positions/orientations */
typedef float 	NOAMAT[3][3];	/* 3x3 matrix for orientations */
typedef float 	VECTOR[3];	/* vector for position */

/*
 * Data transmission format numbers :
 */
#define	COM_FMT00	0
#define	COM_FMT01	1
#define	COM_FMT02	2
#define	COM_FMT03	3
#define	COM_FMT04	4
#define	COM_FMT05	5
#define	COM_FMT06	6
#define	COM_FMT07	7
#define	COM_FMT08	8
#define	COM_FMT09	9
#define	COM_FMT10	10
#define	COM_FMT11	11

/* 
 *	Arm classification : Control arm number selection
 */
typedef unsigned long	ARM;
#define	ARM0	(ARM)0
#define	ARM1	(ARM)1
#define	ARM2	(ARM)2
#define	ARM3	(ARM)3
#define	ARM4	(ARM)4
#define	ARM5	(ARM)5
#define	ARM6	(ARM)6
#define	ARM7	(ARM)7
#define	ARM8	(ARM)8
#define	ARM9	(ARM)9
#define	ARM10	(ARM)10
#define	ARM11	(ARM)11
#define	ARM12	(ARM)12
#define	ARM13	(ARM)13
#define	ARM14	(ARM)14
#define	ARM15	(ARM)15

/* 
 * Data types when in processing end :
 */
#define	WM_WAIT		0
#define	WM_NOWAIT	1

/* 
 * axis data structure for 6-axis and 7-axis arm
 */
typedef struct {
	float s1;
	float s2;
	float s3; /* not use for 6-axis arm */
	float e1;
	float e2;
	float w1;
	float w2;
}ANGLE, *ANGLEP;


/* 
 *  Teach data structures
 */
typedef struct {
	float	agl[7];
	float	vel[2];
	long	atr[12];
} PLYPNT, *PLYPNTP;

typedef struct {
	PLYPNT	pnt;
	char	cmt[32];
} PLAY, *PLAYP;

/*

*/
typedef struct {
	float	xyz[3];
	float	noa[3][3];
} NOAP, *NOAPP;

/*
 *	JUMP data structures	
 */
typedef struct {
	long	cnd[2];
	long	xdi;
	long	tim;
	long	key;
	long	pid;
	long	cnt;
}JUDGE, *JUDGEP;

typedef struct {
	long	cid;
	JUDGE	jdg[8];
}JUMP, *JUMPP;

typedef struct {
	PLAY	ply;
	NOAP	noa;
	JUMP	jmp;
}PNTDAT, *PNTDATP;

/* 
 * Arm Status Structure : Structure set by the motion controller:
 */
typedef struct {
  long   max;    /* Board controllable arm numbers   1or2 */
  long   arm;    /* Arm identification number 0or1 */
  long   axis;   /* Arm axis numbers */
  long   typ;    /* Arm type */
  long   drv;    /* Servo driver classification */
  long   dio;    /* Extension DIO board      exist  /  not exist */
  long   remote; /* operation mode (valid / invalid) */
  long   count;  /* Control Counter value */
  long   error;	 /* Error code */
  ANGLE  angle;  /* Current axis value */
  MATRIX noap;   /* Current tip orientation matrix */
  float  ypr[3]; /* Current orientation */ /*- ypr : Yaw Pitch Roll -*/
}ARMSTATUS, *ARMSTATUSP;

/* 
 * Digital I/O structure
 */
typedef struct {
  unsigned char io1;	/*DIO 1ch*/
  unsigned char io2;	/*DIO 2ch*/
  unsigned char io3;	/*DIO 3ch*/
  unsigned char io4;	/*DIO 4ch*/
}DIOSTATUS, *DIOSTATUSP;

/* 
 * Parameter structure
 */
typedef struct {
  float	rezl;        /* Resolver resolution */
  float	pul[7];      /* Position limiter (+) */
  float	pdl[7];      /* Position limiter (-) */
  float	vel[7 + 2];  /* Velocity limiter */
  float	dev[7 + 2];  /* Default Velocity */
  float	lim[7 + 2];  /* 				*/
  float	ceh[7 + 2];  /* 				*/
  float	cem[7 + 2];  /* 				*/
  float	cel[7 + 2];  /* 				*/
  float	pg1[7];      /* Position control gain 1 */
  float	pg2[7];      /* Position control gain 2 */
  float	vg1[7];      /* Velocity control gain */
  float	tg1[7];      /* Force control gain */
  float	pcm[7];      /* position control selection matrix */
  float	fcm[7];      /* Force control selection matrix */
  float	arl[7];      /* Arm length */
  float	arg[7];      /* Axis gravity center position */
  float	arw[7];      /* Axis weight */
  float	hom[7];      /* Home position recovery target value */
  float	saf[7];      /* Safety position recovery target value */
  float	esc[7];      /* Escape position recovery target value */
  float	tol[7];      /* Tool parameter */
  float	fvl[7];      /* 				*/
  long	dmy[7];      /* */
  long	spa[7];      /* Spare */
} PARAM, *PARAMP;

/* 
 *	Axis classification : Control axis number selection :
 */
typedef unsigned long	AXIS;
#define	S1	(AXIS)0x01
#define	S2	(AXIS)0x02
#define	S3	(AXIS)0x04
#define	E1	(AXIS)0x08
#define	E2	(AXIS)0x10
#define	W1	(AXIS)0x20
#define	W2	(AXIS)0x40

#define	AXISALL		(S1|S2|S3|E1|E2|W1|W2)
#define	ALLAXIS		(S1|S2|S3|E1|E2|W1|W2)
#define	LOCKAXIS_S1	(   S2|S3|E1|E2|W1|W2)
#define	LOCKAXIS_S3	(S1|S2|   E1|E2|W1|W2)

/*
 *    Servo driver classification : Control servo driver number selection :
 */
typedef unsigned long	DRIVER;
#define	DRV1	(DRIVER)0   /* servo driver 1 (S1, S2) */
#define	DRV2	(DRIVER)1   /* servo driver 2 (S3, E1) */
#define	DRV3	(DRIVER)2   /* servo driver 3 (E2, W1) */
#define	DRV4	(DRIVER)3   /* servo driver 4 (W2) */

/*	
 *	Playback motion classification :
 */
typedef unsigned long	PLAYBACK;
#define	PB_FORES	(PLAYBACK)0
#define	PB_BACKS	(PLAYBACK)1
#define	PB_FORE		(PLAYBACK)2
#define	PB_BACK		(PLAYBACK)3

/*
 *	Teach data deletion operation classification :
 */
typedef unsigned long	PNTDEL;
#define	PD_CUR		(PNTDEL)0x7500	/* Current point teach data deletion */
#define	PD_FORE		(PNTDEL)0x7501	/* Previous current point
                                            teach data deletion */
#define	PD_ALL		(PNTDEL)0x7502	/* All active teach data deletion */
#define	PD_ALLDATA	(PNTDEL)0	/* All teach data deletion */

/*
 *	Teach data attribution alteration classification
 */
typedef unsigned long	PNTATTR;
#define	PA_CHGVEL	(PNTATTR)0x7300 /* Linear velocity alteration */
#define	PA_CHGWAIT	(PNTATTR)0x7301 /* Wait time alteration */
#define	PA_VELPTN	(PNTATTR)0x7302	/* Velocity interpolation */
#define	PA_ROTVEL	(PNTATTR)0x7303	/* Rotational velocity alteration */
#define	PA_AXSACC	(PNTATTR)0x7304	/* Each axis precision */
#define	PA_RMRCACC	(PNTATTR)0x7305	/* Straight line precision */
#define	PA_JUMPID	(PNTATTR)0x7306	/* JUMP conditional number */

/*
 *	Teach data type classification
 */
typedef unsigned long	PNTTYPE;
#define	PT_CP	 (PNTTYPE)0x7100 /* Not available */
#define	PT_PTP	 (PNTTYPE)0x7101 /* Loading axis value for linear 
                                     interpolation */
#define	PT_BCP	 (PNTTYPE)0x7102 /* Not available */
#define	PT_BPTP	 (PNTTYPE)0x7103 /* Linear interpolation axis value insertion*/
#define	PT_ARC1	 (PNTTYPE)0x7104 /* Arc 1st point axis value loading */
#define	PT_ARC2	 (PNTTYPE)0x7105 /* Arc 2nd point axis value loading */
#define	PT_ARC3	 (PNTTYPE)0x7106 /* Arc 3rd point axis value loading */
#define	PT_CIR1	 (PNTTYPE)0x7107 /* Circle 1st point axis value loading */
#define	PT_CIR2	 (PNTTYPE)0x7108 /* Circle 2nd point axis value loading */
#define	PT_CIR3	 (PNTTYPE)0x7109 /* Circle 3rd point axis value loading */
#define	PT_AXS	 (PNTTYPE)0x710a /* Loading axis value for axis interpolation*/
#define	PT_BAXS	 (PNTTYPE)0x710b /* Inserts axis value for axis interpolation*/
#define	PT_POS	 (PNTTYPE)0x710c /* Loading NOAP for linear interpolation */
#define	PT_BPOS	 (PNTTYPE)0x710d /* Inserts NOAP for linear interpolation */
#define	PT_ARC4	 (PNTTYPE)0x710e /* Arc 1st point NOAP loading */
#define	PT_ARC5	 (PNTTYPE)0x710f /* Arc 2nd point NOAP loading */
#define	PT_ARC6	 (PNTTYPE)0x7110 /* Arc 3rd point axis value loading */
#define	PT_CIR4	 (PNTTYPE)0x7111 /* Circle 1st point NOAP loading */
#define	PT_CIR5	 (PNTTYPE)0x7112 /* Circle 2nd point NOAP loading */
#define	PT_CIR6	 (PNTTYPE)0x7113 /* Circle 3rd point NOAP loading */

/*
 *	Teach data pointer operation classification
 */
typedef unsigned long	PNTMOVE;
#define	PM_TOP	(PNTMOVE)0x7100	/* Moves pointer to top. */
#define	PM_NEXT	(PNTMOVE)0x7101	/* Pointer forward once. */
#define	PM_PRIV	(PNTMOVE)0x7102	/* Pointer backward once. */
#define	PM_BTM	(PNTMOVE)0x7103	/* Moves pointer to bottom. */
#define	PM_JMP	(PNTMOVE)0x7104	/* Moves pointer to designated number. */
#define	PM_CIR	(PNTMOVE)0x7105	/* Circle Teach point searched, moving
                                    pointer to teach point found first. */
#define	PM_ARC	(PNTMOVE)0x7106	/* Arc teach point searched, moving
                                    pointer to teach point found first. */

/*
 *	Default velocity alteration classification :
 */
typedef unsigned long  VELTYPE;
#define	VT_ONEVEL  (VELTYPE)0  /* Each axis default velocity alteration */
#define	VT_XYZVEL  (VELTYPE)1  /* Tip position default velocity alteration */
#define	VT_YPRVEL  (VELTYPE)2  /* Tip orientation define velocity alteration */

/*
 *	Velocity control mode classification :
 */
typedef unsigned long	VELMODE;
#define	VM_XYZ    (VELMODE)0x200 /* Base coordinate linear velocity control */
#define	VM_YPR    (VELMODE)0x201 /* Base coordinate rotational
                                     velocity control */
#define	VM_xyz    (VELMODE)0x202 /* Mechanical interface coordinate
                                     linear velocity control */
#define	VM_ypr    (VELMODE)0x203 /* Mechanical interface coordinate
                                     rotational velocity control */
#define	VM_ONE    (VELMODE)0x204 /* Each axis velocity control */
#define	VM_XYZYPR (VELMODE)0x205 /* Base coordinate linear/rotational
                                     velocity control */
#define	VM_xyzypr (VELMODE)0x206 /* Mechanical interface coordinate
                                     linear/rotational velocity control */

/*
 *	Redundant axis control mode classification :
 */
typedef unsigned long	JOUMODE;
#define	JM_SET	   (JOUMODE)0X345 /* Redundant axis control
                                      parameter operation start */
#define	JM_RESET   (JOUMODE)0X346 /* Redundant axis control parameter reset */
#define	JM_VSET	   (JOUMODE)0X347 /* Redundant axis velocity control mode */
#define	JM_ON	   (JOUMODE)0X348 /* Redundant axis control all axes
                                      restriction mode */
#define	JM_OFF	   (JOUMODE)0X349 /* restriction axis control
                                      restriction release */
#define	JM_S3ON	   (JOUMODE)0X34a /* Redundant axis control only S3
                                      axis interpolation restriction mode */
#define	JM_S3DIV   (JOUMODE)0X34b /* Redundant axis control S3 axis
                                      interpolation restriction mode */
#define	JM_S3HOLD  (JOUMODE)0X34c /* Redundant axis control S3 axis
                                      fixation restriction mode */

/*
 *	Ĺ⡼
 */
typedef unsigned long	JOUTYPE;
#define	JT_RIGHT	(JOUTYPE)1  /* Moves redundant axis restriction
                                        parameter to the right */
#define	JT_HOLD		(JOUTYPE)0  /* Holds redundant axis
                                        restriction parameter */
#define	JT_LEFT		(JOUTYPE)-1 /* Moves redundant axis restriction
                                        parameter to the left */

/*
 *	Target tip matrix control mode classification :
 */
typedef unsigned long	MOVEMODE;
#define	MM_XYZ		(MOVEMODE)0x5680 /* Tip position control */
#define	MM_NOA		(MOVEMODE)0x5681 /* Tip orientation control */
#define	MM_XYZNOA	(MOVEMODE)0x5682 /* Tip position/orientation control */

/*
 *	Direct control classification (optional function)
 */
typedef unsigned long	DIRECTMODE;
#define	DM_STOP		(DIRECTMODE)0 /* Direct control stop */
#define	DM_START	(DIRECTMODE)1 /* Direct control start */

#define	ARM_STANDING	 1	/* Floor mounted */
#define	ARM_HANGING	-1	/* Suspending from ceiling */

/*
 *	DIO port numbers :
 */
typedef unsigned long	DIOPORT;
#define	DP_PORT1	(DIOPORT)0   /* DIO 0  port selection */
#define	DP_PORT2	(DIOPORT)1   /* DIO 1  port selection */
#define	DP_PORT3	(DIOPORT)2   /* DIO 2  port selection */
#define	DP_PORT4	(DIOPORT)3   /* DIO 3  port selection */
#define	DPO_PORT1	(DIOPORT)4   /* DO 1 port selection */
#define	DPO_PORT2	(DIOPORT)5   /* DO 2 port selection */
#define	DPO_PORT3	(DIOPORT)6   /* DO 3 port selection */
#define	DPO_PORT4	(DIOPORT)7   /* DO 4 port selection */
#define	DPX_PORT1	(DIOPORT)8   /* DO 1 port selection ??? */
#define	DPX_PORT2	(DIOPORT)9   /* DO 2 port selection ??? */
#define	DPX_PORT3	(DIOPORT)10  /* DO 3 port selection ??? */
#define	DPX_PORT4	(DIOPORT)11  /* DO 4 port selection ??? */

/*
 *	DIO channel numbers :
 */
typedef unsigned long	DIOCH;
#define	DC_CH1	(DIOCH)0  /* Channel 1 selection */
#define	DC_CH2	(DIOCH)1  /* Channel 2 selection */
#define	DC_CH3	(DIOCH)2  /* Channel 3 selection */
#define	DC_CH4	(DIOCH)3  /* Channel 4 selection */
#define	DC_CH5	(DIOCH)4  /* Channel 5 selection */
#define	DC_CH6	(DIOCH)5  /* Channel 6 selection */
#define	DC_CH7	(DIOCH)6  /* Channel 7 selection */
#define	DC_CH8	(DIOCH)7  /* Channel 8 selection */

/*
 *	Sensor correction coordinate classification :
 */
typedef unsigned long		TRANSMODE;
#define	MODE_xyz   (TRANSMODE)0x01  /* Adds absolute correction value in the
                                     mechanical interface coordinate system */
#define	MODEIxyz   (TRANSMODE)0x02  /* Adds relative correction value in the
                                     mechanical interface coordinate system */
#define	MODE_XYZ   (TRANSMODE)0x04  /* Adds absolute correction value in the
                                     base coordinate system */
#define	MODEIXYZ   (TRANSMODE)0x08  /* Adds relative correction value in the
                                     base coordinate system */
#define	MODE_wave  (TRANSMODE)0x10  /* Adds absolute correction value in the
                                     trajectory coordinate system */
#define	MODEIwave  (TRANSMODE)0x20  /* Adds relative correction value in the
                                     trajectory coordinate system */

/*
 *	Teach point attribute designation :
 */
typedef unsigned long	PNTID;
#define	PA_SETID	(PNTID)0x7304

/*
 *	Circle and arc teach point number designation :
 */
typedef unsigned long	PNTNO;
#define	PN_1 	(PNTNO)1
#define	PN_2 	(PNTNO)2
#define	PN_3 	(PNTNO)3

/*
 *      JUMP data valid/invalid (in teach data)
 */
typedef unsigned long	JUMPONOFF;
#define JMP_ON     (JUMPONOFF)1		/* ON   Valid */
#define JMP_OFF    (JUMPONOFF)0		/* OFF  Invalid */

/*
 *	JUMP data valid/invalid (in JUMP data)
 */
typedef unsigned long	JUMPENABLEDISABLE;
#define	JMPENABLE 	(JUMPENABLEDISABLE)0x01000000  /* Valid */
#define	JMPDISABLE 	(JUMPENABLEDISABLE)0x00000000  /* Invalid */

/*
 *	JUMP Command
 */
typedef unsigned long	JUMPORDER;
#define	NO_JUMP		(JUMPORDER)0x00010000  /* Unconditional JUMP */
#define	DI_JUMP 	(JUMPORDER)0x00020000  /* DI conditional JUMP */
#define	DI_WAITJUMP 	(JUMPORDER)0x00030000  /* DI conditional WAITJUMP */
#define	DI_WAIT 	(JUMPORDER)0x00040000  /* DI conditional WAIT */

/*
 *	JUMP Conditional Logic
 */
typedef unsigned long	JUMPDILOGIC;
#define	LEVEL_ON 	(JUMPDILOGIC)0x00000100
#define	LEVEL_OFF	(JUMPDILOGIC)0x00000200
#define	EDGE_ON 	(JUMPDILOGIC)0x00000400
#define	EDGE_OFF	(JUMPDILOGIC)0x00000800

/*
 *	JUMP ticket-oriented DI
 */
typedef unsigned long	DIOKIND;
#define	DIO_INTERNAL	(DIOKIND)0x00000000  /* System */
#define	DIO_EXTERNAL	(DIOKIND)0x00000001  /* User */

/*
 *  Teaching place when in CUBE creation :
 */
typedef unsigned long	CUBEPNT;
#define MAXPNT    (CUBEPNT)1
#define MINPNT    (CUBEPNT)2
#define CENTERPNT (CUBEPNT)3

/*
 *	mask setting
 */
typedef unsigned long	DIOMASK;
#define	DIMSK	 	(DIOMASK)0
#define	DOMSK		(DIOMASK)1

/*
 *  RETRAC ON/OFF
 */
typedef unsigned long	RETRAC;
#define RETRACOFF    (RETRAC)0
#define RETRACON     (RETRAC)1

/*
 *  CUBE information :
 */
typedef unsigned long	CUBEINFO;
#define NOCUBE       (CUBEINFO)0x00000000
#define CUBEON       (CUBEINFO)0x00000001
#define CUBEMAX      (CUBEINFO)0x00000002
#define CUBEMIN      (CUBEINFO)0x00000004
#define CUBECENTER   (CUBEINFO)0x00000008
#define CUBESIDE     (CUBEINFO)0x00000010


/*
 *	TEACH MODE
 */
typedef unsigned long	TEACHMODE;
#define	TEACH_OFF 	(TEACHMODE)0
#define	TEACH_LOW	(TEACHMODE)1
#define	TEACH_MID	(TEACHMODE)2
#define	TEACH_HIGH	(TEACHMODE)3

/*
 *  TEACH LOCK
 */
typedef unsigned long	TEACHLOCK;
#define LOCK_OFF    (TEACHLOCK)0
#define LOCK_ON     (TEACHLOCK)1

/*
 *    Communication status with servo driver :
 */
typedef unsigned long	COMSTATUS;
#define STP_STATUS    (COMSTATUS)0
#define MOV_STATUS    (COMSTATUS)1
#define SIM_STATUS    (COMSTATUS)2

/*
 *    for RETRAC :
 */
#define	MOD_ROBFILE	1
#define	MOD_TOLFILE	2

/*
 *   Sensor correction data structure :
 */
typedef struct {
	long	Enable;		/* Designation bit */
	float	_xyz[3];	/* Mechanical interface coordinate absolute
                                    deviation correction value */
	float	Ixyz[3];	/* Mechanical interface coordinate relative
                                    deviation correction value */
	float	_XYZ[3];	/* Base coordinate absolute deviation 
                                    correction value */
	float	IXYZ[3];	/* Base coordinate relative deviation 
                                    correction value */
	float	_wave[3];	/* Trajectory coordinate absolute deviation
                                    correction value */
	float	Iwave[3];	/* Trajectory coordinate relative deviation
                                    correction value */
} TRANSMAT, *TRANSMATP;

/* 
 *   Arm target value structure :
 */
typedef struct {
	ANGLE		angle;   /* Target value */
	MATRIX		noap;    /* Tip position/orientation matrix */
	float		ypr[3];  /* Tip position */
} ARMTARGET, *ARMTARGETP;

/*
 *   Structure to send commands from the motion control to the servo driver :
 */
typedef struct {
	long	sig;
	long	trq;
	long	vel;
} O8DRIVE;

/*
 *   Structure to send commands from the servo driver to the motion control :
 */
typedef struct {
	long	sts;
	long	agl;
	long	vel;
	long	trq;
} I8DRIVE;

/*
 *	for Dead man switch :
 */
#define SET_DDM	3


/*
 *    CUBE information structure
 */
typedef struct{
	long	ena;
	long	mod;
	float	max[3];
	float	min[3];
	char	cmt[32];
} CUBE, *CUBEP;


/* 
 *    Debug structure :
 */
typedef struct {
	long	ldbg[16];
	float	fdbg[32];
} DEBG, *DEBGP;

/*===========================================================================
         Function list
===========================================================================*/
#define	DllExport	
#define	__stdcall	

#ifdef __cplusplus
extern  "C" {
#endif


  /*
   *	System Setting & Initialization Function
   */
  long pa_ini_sys(void); /* PA library initialization */
  long pa_ter_sys(void); /* PA library termination */

  /*
   *	Arm status control function
   */

  long pa_opn_arm(ARM);       /* Open arm (control arm selection) */
  long pa_con_arm(ARM,char*); /* ???  Call WinLANinit that do gethostbyname()
			       *  and connect() ...  Not documented */ 
  long pa_cls_arm(ARM);       /* Close arm  (control arm separation) */

  long pa_sta_arm(ARM);       /* Controller operation Start (Servo
				 driver communication start) */ 
  long pa_ext_arm(ARM);       /* Controller operation end (Servo
				 driver communication end) */

  long pa_stp_arm(ARM,long);  /* Arm brake-stop */
  long pa_sus_arm(ARM,long);  /* Arm temporarily stop */
  long pa_rsm_arm(ARM,long);  /* Arm temporarily-stop-release */
  /*
   *    Arm motion control function
   */
  long pa_exe_axs(ARM,AXIS,ANGLEP,long); /* Axis angle control */
  long pa_exe_edr(ARM,AXIS,ANGLEP,long); /* ??? */

  long pa_exe_hom(ARM,long);  /* Axis angle control to home position */
  long pa_exe_esc(ARM,long);  /* Axis angle control to escape position */
  long pa_exe_saf(ARM,long);  /* Axis angle control to safety position */

  long pa_mov_XYZ(ARM,float,float,float,long); /* Position deviation control 
						  in robot coordinate system */
  long pa_mov_YPR(ARM,float,float,float,long); /* Orientation deviation
						  control in robot
						  coordinate system */
  long pa_mov_xyz(ARM,float,float,float,long); /* Position deviation control
						  in tip coordinate system
						  (available only for 
						  visual C++) */
  long pa_mov_ypr(ARM,float,float,float,long); /* Orientation deviation control
						  in robot coordinate system
						  (available only for 
						  visual C++) ?? */
  long pa_mov_mat(ARM,MOVEMODE,MATRIX,ANGLEP,long);/* Tip position/orientation
						      absolute position
						      control*/ 
/*
 *	Function on teach point operation & playback control
 */
  long pa_chg_pnt(ARM,PNTMOVE,long); /* Teach point pointer alteration
					(current point alteration) */
  long pa_add_pnt(ARM,PNTTYPE);      /* Teach point addition */
  long pa_del_pnt(ARM,PNTDEL);       /* Teach point deletion */
  long pa_rpl_pnt(ARM);              /* Teach point replacement */
  /* ?? */
  long pa_set_pnt(ARM,PNTATTR,long*,float); /* Teach point
					       attrribution setting */
  long pa_mov_pnt(ARM,long);   /* Linear motion control from the
				  present position to the current point */
  /* ?? */
  long pa_ply_pnt(ARM,PLAYBACK,long,long); /* Playback control */
  /* ?? */
  long pa_lod_pnt(ARM,long,char *); /* Loading teach data to controller */
  long pa_sav_pnt(ARM,char *);      /* Saving teach data to
				       man-machine controller */

  long pa_vel_pnt(ARM,float); /* Playback control velocity coefficient
				 alteration */

/*
 *	Velocity control function
 */
  long pa_mod_vel(ARM,VELMODE,AXIS); /* Velocity control mode setting */
  long pa_odr_vel(ARM,float[]);      /* Velocity control data set */

/*
 *	Redundant axis control function (7-axis, only)
 */
  long pa_mod_jou(ARM,JOUMODE); /* Redundant axis control mode setting */
  long pa_odr_jou(ARM,JOUTYPE); /* Redundant axis control data set */

/*
 *	Tip absolute position/orientation axis real-time control function
 */
  long pa_mod_dpd(ARM);               /* Target position/orientation 
					 real-time control mode setting */
  long pa_odr_dpd(ARM,MATRIX,ANGLEP); /* Target position/orientation
					 real-time control data set */

/*
 *	??
 */
  long pa_wet_ded(ARM,AXIS);     /* Weight conpensation control */
  long pa_get_amp(ARM);          /* ??? Not documented */
  long pa_mod_dir(ARM,DIRECTMODE); /* Servo lock ON/OFF when in direct
				      control start */

/*
 *	Function on position setting/definition
 */
  long pa_set_hom(ARM,ANGLEP); /* Home position setting */
  long pa_set_esc(ARM,ANGLEP); /* Escape position setting */
  long pa_set_saf(ARM,ANGLEP); /* Safety position setting */

  long pa_def_hom(ARM); /* Defining current value as home position */
  long pa_def_esc(ARM); /* Defining current value as escape position */
  long pa_def_saf(ARM); /* Defining current value as safety position */

/*
 *	??
 */
  long pa_set_mtx(ARM,MATRIX); /* Coordinate spatial conversion matrix
				  (position offset) setting (for playback 
				  control) */

/*
 *	??
 */
  long pa_get_sts(ARM,ARMSTATUSP); /* Current arm information loading */
  long pa_get_cnt(ARM,long*);      /* Current arm control counter loading */
  long pa_get_err(ARM,long*);      /* Current arm error information loading */
  long pa_get_agl(ARM,ANGLEP);     /* Current arm axis value loading */
  long pa_get_xyz(ARM,VECTOR);     /* Current arm tip position loading */
  long pa_get_noa(ARM,MATRIX);     /* Current arm orientation matrix loading */
  long pa_get_ypr(ARM,VECTOR);     /* Current arm position angle loading */
  long pa_get_prm(ARM,PARAMP);     /* Current arm parameter loading */
  long pa_get_pnt(ARM,PNTDATP);    /* Current point teach point data loading */
  long pa_get_cur(ARM,long*);    /* Current point teach point number loading */
  long pa_get_num(ARM,long*);       /* Teach point all numbers loading */
  long pa_get_lvl(ARM,long,long*);  /* ??? not documented ??? */
  long pa_get_fvl(ARM,long,float*); /* ??? not documented ??? */

/*
 *    Digital input/output function
 */
  long pa_inp_dio(ARM,DIOKIND,DIOSTATUSP); /* Digital input
					      (32ch. unit input */
  long pa_oup_dio(ARM,DIOKIND,DIOSTATUSP); /* Digital output
					      (32ch. unit output */
  long pa_get_dio(ARM,DIOKIND,DIOPORT,DIOCH,unsigned char*); /* Digital input 
								(1ch. unit 
								input */ 
  long pa_set_dio(ARM,DIOKIND,DIOPORT,DIOCH); /* Digital output 
						 (1ch. unit set*/ 
  long pa_rst_dio(ARM,DIOKIND,DIOPORT,DIOCH); /* Digital output 
						 (1ch. unit reset) */

/*
 *	Function on parameter
 */
  long pa_set_tol(ARM,float,float,float,float); /* Tool information setting */
  long pa_set_vel(ARM,VELTYPE,float*); /* Default velocity alteration */
  long pa_lod_ctl(ARM,char *); /* loading parameter to controller */

  long pa_sav_ctl(ARM,char *);		/* ?? */

  long pa_lod_def(ARM);   /* ?? */
  long pa_sav_def(ARM);   /* ?? */

/*
 *	??
 */
  long pa_map_ctl(ARM);  /* Shared area mapping with controller */
  short pa_fsh_chk(ARM); /* Waiting for control command processing
			    completion */
  long pa_req_ctl(ARM,long); /* Issuing command setting intrusion to
				controller */
  long pa_rst_ctl(ARM); /* Arm error information reset */

  long __stdcall pa_err_mes(long,char*); /* Error message acquisition */

/*======== ?? ========*/
/*
 *	??
 */
  long pa_sta_sim(ARM);  /* Simulation control start (simulation
			    communication start) */
  long pa_ext_sim(ARM);  /* Simulation control end (simulation
			    communication end) */

/*
 *	??
 */
  long pa_swt_dio(ARM,long);  /* Playback control teach point DO
				 valid/invalid setting */
  long pa_chg_dio(ARM,DIOSTATUSP); /* Teach point (PTP) DO attribution
				      setting */
/*
 *	??
 */
  long pa_set_idn(ARM,PNTID,long); /* ID_No setting at teach point */
  long pa_get_idn(ARM,long*);      /* Current point ID_No. loading */
  long pa_axs_pnt(ARM,long); /* Axis motion control from the present
				position to the current point */
  long pa_set_dlc(ARM,long);  /* Playback DO automatic stop/non stop setting */
  long pa_get_dlc(ARM,long*); /* Playback DO automatic stop/non stop loading */

/*
 *	??
 */
  long   pa_odr_xyz(ARM,TRANSMATP); /* Tip position offset value setting */
  long   pa_lmt_xyz(ARM,float);     /* Limit value setting when in
				       offset value supplement */
  long   pa_get_lmt(ARM,float*);    /* Limit value loading when in
				       offset value supplement */
  short pa_fsh_sub(ARM); /* Waiting for control command processing
			    completion */
  long   pa_req_sub(ARM,long); /* Issuing command setting intrusion to
				controller */

/*
 *    ??
 */
  long pa_set_mat(ARM,MATRIX,MATRIX); /* Coordinate spatial conversion
					 matrix setting */

/*
 *     ??
 */
  long pa_drt_ded(ARM,long);/* Arm installation direction setting */
  long pa_chk_cnt(ARM);     /* Synchronisation processing in direct control */
  long pa_set_tim(ARM,long);/* Time-out loading in synchronisation processing*/

/*
 *     Arm status information loading function
 */
  long pa_get_mod(ARM,long*); /* Arm control status loading */
  long pa_get_jou(ARM,long*); /* Arm redundant axis control mode loading 
				 Redundant axis control function
				 (7-axis, only) [Suite] */
  long pa_get_ver(ARM,float*); /*Motion controller S/W version number loading*/
  long pa_get_pvl(ARM,float*); /* Playback control velocity
				  coefficient loading*/
  long pa_get_pdo(ARM,long*);  /* Playback control teach point DO
				  valid/invalid loading */
  long pa_get_cpt(ARM,PNTNO,PNTDATP); /* Current point circle/arc teach
				       data loading */
  long pa_get_mat(ARM,MATRIX,MATRIX); /* Current setting conversion
					 matrix loading */
  long pa_get_sns(ARM,TRANSMATP);     /* Current setting tip offset
					 value loading */
  long pa_get_tar(ARM,ARMTARGETP); /* Current arm target data loading */
  long pa_get_com(ARM,long*); /* Communication status (no
				 communication/simulation actual machine)
				 loading */ 
  long pa_get_tim(ARM,long*); /* Time out loading in synchronisation
				 processing */
  long pa_get_drt(ARM,long*); /* Arm installation direction
				 acquisition/loading */

/*
 *	Redundant axis control function (7-axis, only) [Suite]
 */
  long pa_mov_jou(ARM,float,long); /* Redundant axis (elbow) motion control */
/*
 *     ??
 */
  long pa_mod_axs(ARM);        /* Axis real-time control mode setting */
  long pa_odr_axs(ARM,ANGLEP); /* Axis real-time control data set */

  /*======== ?? ========*/
  long pa_ply_set(ARM,long,long*); /* Teach data Key acquisition by
				      number designation */
  long pa_act_pnt(ARM,long);       /* Active teach data switching */
  long pa_jmp_set(ARM,long,long,JUMPP); /* JUMP data acquisition by
					   number designation */
  long pa_get_jmp(ARM,long,long,JUMPP); /* JUMP data acquisition by
					   Key/ID designation*/
  long pa_set_jmp(ARM,long,long,JUMPP); /* JUMP data setting */
  long pa_ena_jmp(ARM,long);            /* JUMP condition
					   valid/invalid setting */
  long pa_ply_mod(ARM,long);            /* Teach mode setting */
  long pa_set_cub(ARM,long,float[],float[]); /* CUBE designation */
  long pa_get_cub(ARM,long,long);            /* CUBE teach designation */
  long pa_cub_len(ARM,long,float[]); /* CUBE side length designation */
  long pa_cub_cmt(ARM,long,char*);   /* Naming CUBE */
  long pa_del_cub(ARM,long);         /* CUBE deletion */
  long pa_ena_cub(ARM,long,long);    /* CUBE valid/invalid */
  long pa_inf_cub(ARM,long,CUBEP);   /* CUBE information reference */
  long pa_get_prj(ARM,char*); /* Project name acquisition */
  long pa_set_prj(ARM,char*); /* Project name setting */
  long pa_sav_prj(ARM,char*); /* Saving project */
  long pa_lod_prj(ARM,char*); /* Loading project */
  long pa_get_sav(ARM,long*); /* Axis Servo ON/OFF status acquisition */
  long pa_chg_key(ARM,long);  /* Current active teach data Key alteration */
  long pa_get_key(ARM,long*); /* Current active teach data Key acquisition */
  long pa_mon_pnt(ARM,PNTDATP); /* Acquired to monitor teach data status */
  long pa_sav_sts(ARM,long[]);  /* Servo status acquisition */
  long pa_set_cmt(ARM,char*); /* Comment setting */
  long pa_tst_nom(ARM,long);  /* RETRAC creation ON/OFF setting */
  long pa_get_rmd(ARM,long*); /* RETRAC creation ON/OFF acquisition */
  long pa_dio_msk(ARM,long,long,long);  /* DIO mask setting */
  long pa_get_msk(ARM,long,long,long*); /* DIO mask acquisition */
  long pa_jmp_cmt(ARM,long,char*); /* Current point shifting by comment */
  long pa_get_ena(ARM,long*); /* JUMP condition valid/invalid acquisition */
  long pa_get_pmd(ARM,long*); /* Teach mode acquisition */
  long pa_sav_ptj(ARM,char*); /* Saving teach data and JUMP data */
  long pa_lod_ptj(ARM,char*); /* Loading teach data and JUMP data */
  long pa_get_dbg(ARM,DEBGP); /* ?? */
  long pa_set_ddm(ARM,long,long);  /* Dead man SW valid/invalid */
  long pa_get_ddm(ARM,long,long*); /* Dead man SW valid/invalid status
				      acquisition */
  long pa_del_jmp(ARM,long,long); /* JUMP data deletion */
  long pa_set_lok(ARM,long);  /* TEACH LOCK setting */
  long pa_get_lok(ARM,long*); /* TEACH LOCK acquisition */
  long pa_tct_tim(ARM,long*); /* Tact time (playback time) acquisition */
  long pa_get_smd(ARM,long*); /* TEACH MODE acquisition from servo */
  long pa_get_max(ARM,long*); /* Board controllable arm number acquisition */
  long pa_get_spt(ARM,long*); /* Acquiring arm identification number */




/*======== H8.4.12 ========*/
/*======== H9.7.18 ========*/
/*
 *	?? pa_mem.c
 */
  long pa_mem_cpy(long,void*,void*,size_t,int); /* ?? */
  long pa_mem_blk(long,void*,void*,size_t,int); /* ?? */
  long pa_cpy_flt(long,void*,void*,int);        /* float ?? */
  long pa_cpy_lng(long,void*,void*,int);        /* long ?? */
  long pa_cpy_uby(long,void*,void*,int);        /* unsigned char ?? */
  long pa_ret_lng(long,void*);                  /* long ?? */
/* pa_egt.c */
  long __stdcall pa_get_lgh(ARM,float*);        /* S2-W2 ?? */

long WinLANinit(long,char*);
long WinLANabort(long);

long WinRTinit(long);
long WinRTabort(long);
long MakeMutex(long);
long DestroyMutex(long);
long LockMutex(long);
long UnlockMutex(long);

  long pa_set_sim(ARM, long);        /* Simulation magnification setting */
  long pa_set_inc(ARM, float);       /* Real-time velocity setting */
  long pa_get_sim(ARM, long*);       /* Simulation magnification acquisition */
  long pa_get_inc(ARM, float*);      /* Real-time velocity acquisition */
  long pa_lod_rob(ARM armno,char *file); /* Robot model file loading */
  long pa_lod_tol(ARM armno,char *file); /* Tool model file loading */
  long pa_sav_rob(ARM armno);        /* model file saving */
  long pa_ena_nom(ARM armno,long sw);/* RETRAC calculation switching */
  long pa_ini_axs(ARM,AXIS,ANGLEP,long); /* ?? */
  long pa_get_nom(ARM armno,long *rpm); /* Acquiring either T-matrix
					   or RETRAC calculation */



#ifdef __cplusplus
}
#endif


#endif	/* _PA_H_	*/



