/**
 ** qarmCntrlTaskCodels.c
 **
 ** Codels used by the control task qarmCntrlTask
 **
 ** Author: Daniel Sidobre
 ** Date: 2004
 **
 **/

#ifdef VXWORKS
# include <vxWorks.h>
#else
# include <portLib.h>
#endif
#include <stdio.h>
#include "server/qarmHeader.h"

#include "qarmPA10.h"

/*------------------------------------------------------------------------
 * qarmStopCntrl  -  control codel of CONTROL request Stop
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmStopCntrl(int *report)
{
  /* controller operation end, servo driver communication end */
  if (qarmEmergencyStop() != OK) {
    *report = S_qarm_PALIB_ERROR; /* message a revoir */
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmStopSoftCntrl  -  control codel of CONTROL request StopSoft
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmStopSoftCntrl(int *report)
{
  if (qarmStopSoft() != OK){
    *report = S_qarm_PALIB_ERROR; /* message a revoir */
    return ERROR ;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmSimulationCntrl  -  control codel of CONTROL request Simulation
 *
 * Description:    
 * Report: OK
 *                 S_qarm_NotInInitialMode
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmSimulationCntrl(QARM_SIMULATION_ENUM *simulation, int *report)
{
  if (SDI_F->state.mode == QARM_STATE_INITIAL) {
    /* do nothing */
  } else {
    *report = S_qarm_NotInInitialMode;
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmManualModeCntrl  -  control codel of CONTROL request ManualMode
 *
 * Description:    
 * Report: OK
 *                 S_qarm_NotInInitialMode
 *                 S_qarm_TrackingActive
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmManualModeCntrl(int *report)
{
  long status;
  ANGLE angle = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

  switch (SDI_F->state.mode) {
  case QARM_STATE_INITIAL: break;
  case QARM_STATE_MANUAL:  return OK;
  case QARM_STATE_TRACK:
    *report = S_qarm_TrackingActive; /* We don't stop tracking here, it's 
				      *  supposed to be stopped by genom
				      *  incompatibility of request */
    return OK;
  case QARM_STATE_FATAL_ERROR:
  default:
    *report = S_qarm_NotInInitialMode;
    QARM_ERROR_PRINT(ERROR, "qarmManualModeCntrl: pa_get_agl");
    return ERROR;
  }
  status = qarmStartIfNeed();
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: qarmStartIfNeed");
    return ERROR;
  }
   /* get q axis angles */
  status = pa_get_agl(PA10_ARM_NUMBER, &angle);
  PA10_ERROR_RETURN(status, "qarmManualModeCntrl pa_get_agl", ERROR);
  /* control at the current position */
  status = pa_exe_axs(PA10_ARM_NUMBER, ALLAXIS, &angle, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "qarmManualModeCntrl pa_exe_axs", ERROR);
  SDI_F->state.mode = QARM_STATE_MANUAL;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmSpeedLimitSetCntrl  -  control codel of CONTROL request SpeedLimitSet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_OutOfRange
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmSpeedLimitSetCntrl(double *speedLimit, int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * qarmSpeedLimitActivateCntrl  -  control codel of CONTROL request SpeedLimitActivate
 *
 * Description:    
 * Report: OK
 *                 S_qarm_OutOfRange
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmSpeedLimitActivateCntrl(QARM_ON_OFF_ENUM *speedLimitMode, int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * qarmGotoQCntrl  -  control codel of CONTROL request GotoQ
 *
 * Description:    
 * Report: OK
 *                 S_qarm_Q_LIMITED
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmGotoQCntrl(Gb_q6 *qConsigne, int *report)
{
  long status;
  ANGLE angle;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmGotoQCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return END;
  }
  angle.s1 = qConsigne->q1;
  angle.s2 = qConsigne->q2;
  angle.s3 = 0.;
  angle.e1 = qConsigne->q3;
  angle.e2 = qConsigne->q4;
  angle.w1 = qConsigne->q5;
  angle.w2 = qConsigne->q6;
  /*  */
  status = pa_exe_axs(PA10_ARM_NUMBER, ALLAXIS, &angle, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_axs", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmGotoQWaitCntrl  -  control codel of EXEC request GotoQWait
 *
 * Description:    
 * Report: OK
 *                 S_qarm_Q_LIMITED
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmGotoQWaitCntrl(Gb_q6 *qConsigne, int *report)
{
  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmGotoQCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return END;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmGotoQRelativeCntrl  -  control codel of CONTROL request GotoQRelative
 *
 * Description:    
 * Report: OK
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *                 S_qarm_Q_LIMITED
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmGotoQRelativeCntrl(Gb_q6 *deltaQ, int *report)
{
  long status;
  ANGLE angle;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmGotoQRelativeCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return END;
  }
  angle.s1 = SDI_F->pos.q.q1 + deltaQ->q1;
  angle.s2 = SDI_F->pos.q.q2 + deltaQ->q2;
  angle.s3 = 0.;
  angle.e1 = SDI_F->pos.q.q3 + deltaQ->q3;
  angle.e2 = SDI_F->pos.q.q4 + deltaQ->q4;
  angle.w1 = SDI_F->pos.q.q5 + deltaQ->q5;
  angle.w2 = SDI_F->pos.q.q6 + deltaQ->q6;
  /* control at the current position */
  status = pa_exe_axs(PA10_ARM_NUMBER, ALLAXIS, &angle, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_axs", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmQRelativeSpeedCntrl  -  control codel of EXEC request QRelativeSpeed
 *
 * Description:    
 * Report: OK
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *                 S_qarm_Q_LIMITED
 *                 S_qarm_NOT_WRITTEN
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmQRelativeSpeedCntrl(QARM_QRELATIVE_STR *delta, int *report)
{
  *report = S_qarm_NOT_WRITTEN;
  QARM_ERROR_PRINT(ERROR, "Normal mode not written");
  return END;
}

/*------------------------------------------------------------------------
 * qarmPa10GotoHomeCntrl  -  control codel of CONTROL request Pa10GotoHome
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *                 S_qarm_ModeError
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10GotoHomeCntrl(int *report)
{
  /* version test : marche en MANUAL_MODE */
  long status;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmTestGotoHomeCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return END;
  }
  status = pa_exe_hom(PA10_ARM_NUMBER, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_hom", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10GotoSafeCntrl  -  control codel of CONTROL request Pa10GotoSafe
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *                 S_qarm_ModeError
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10GotoSafeCntrl(int *report)
{
  /* version test : marche en MANUAL_MODE */
  long status;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmTestGotoSafeCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return ERROR;
  }
  status = pa_exe_saf(PA10_ARM_NUMBER, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_saf", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10GotoEscapeCntrl  -  control codel of CONTROL request Pa10GotoEscape
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *                 S_qarm_ModeError
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10GotoEscapeCntrl(int *report)
{
  /* version test : marche en MANUAL_MODE */
  long status;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmTestGotoEscapeCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE; 
    return ERROR;
  }
  status = pa_exe_esc(PA10_ARM_NUMBER, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_esc", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10StatusGetCntrl  -  control codel of CONTROL request Pa10StatusGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10StatusGetCntrl(int *report)
{
  ARMSTATUS armstatus;
  long status;

  status = pa_get_sts(PA10_ARM_NUMBER, &armstatus);
  PA10_ERROR_RETURN(status, "qarmGetArmStatusCntrl: pa_get_sts", END);
  SDI_F->state.iarm.armStatus.max    = armstatus.max;
  SDI_F->state.iarm.armStatus.arm    = armstatus.arm;
  SDI_F->state.iarm.armStatus.axis   = armstatus.axis;
  SDI_F->state.iarm.armStatus.typ    = armstatus.typ;
  SDI_F->state.iarm.armStatus.drv    = armstatus.drv;
  SDI_F->state.iarm.armStatus.dio    = armstatus.dio;
  SDI_F->state.iarm.armStatus.remote = armstatus.remote;
  SDI_F->state.iarm.armStatus.count  = armstatus.count;
  SDI_F->state.iarm.armStatus.error  = armstatus.error;
  SDI_F->state.iarm.armStatus.angle.s1 = armstatus.angle.s1;
  SDI_F->state.iarm.armStatus.angle.s2 = armstatus.angle.s2;
  SDI_F->state.iarm.armStatus.angle.s3 = armstatus.angle.s3;
  SDI_F->state.iarm.armStatus.angle.e1 = armstatus.angle.e1;
  SDI_F->state.iarm.armStatus.angle.e2 = armstatus.angle.e2;
  SDI_F->state.iarm.armStatus.angle.w1 = armstatus.angle.w1;
  SDI_F->state.iarm.armStatus.angle.w2 = armstatus.angle.w2;
  SDI_F->state.iarm.armStatus.noap.vx.x = armstatus.noap[0][0];
  SDI_F->state.iarm.armStatus.noap.vx.y = armstatus.noap[1][0];
  SDI_F->state.iarm.armStatus.noap.vx.z = armstatus.noap[2][0];
  SDI_F->state.iarm.armStatus.noap.vy.x = armstatus.noap[0][1];
  SDI_F->state.iarm.armStatus.noap.vy.y = armstatus.noap[1][1];
  SDI_F->state.iarm.armStatus.noap.vy.z = armstatus.noap[2][1];
  SDI_F->state.iarm.armStatus.noap.vz.x = armstatus.noap[0][2];
  SDI_F->state.iarm.armStatus.noap.vz.y = armstatus.noap[1][2];
  SDI_F->state.iarm.armStatus.noap.vz.z = armstatus.noap[2][2];
  SDI_F->state.iarm.armStatus.noap.vp.x = armstatus.noap[0][3];
  SDI_F->state.iarm.armStatus.noap.vp.y = armstatus.noap[1][3];
  SDI_F->state.iarm.armStatus.noap.vp.z = armstatus.noap[2][3];
  SDI_F->state.iarm.armStatus.ypr[0] = armstatus.ypr[0];
  SDI_F->state.iarm.armStatus.ypr[1] = armstatus.ypr[1];
  SDI_F->state.iarm.armStatus.ypr[2] = armstatus.ypr[2];

  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10CountGetCntrl  -  control codel of CONTROL request Pa10CountGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10CountGetCntrl(int *report)
{
  long status;
  long counter;

  status = pa_get_cnt(PA10_ARM_NUMBER, &counter);
  PA10_ERROR_RETURN(status, "qarmGetArmCountCntrl: pa_get_cnt", END);
  SDI_F->state.iarm.armStatus.count = counter;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10ErrorGetCntrl  -  control codel of CONTROL request Pa10ErrorGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10ErrorGetCntrl(int *report)
{
  long status;
  long pa10Error;

  status = pa_get_err(PA10_ARM_NUMBER, &pa10Error);
  PA10_ERROR_RETURN(status, "qarmGetArmErrorCntrl: pa_get_err", END);
  SDI_F->state.iarm.armStatus.error = pa10Error;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10ModeGetCntrl  -  control codel of CONTROL request Pa10ModeGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10ModeGetCntrl(int *report)
{
  long status;
  long mode;

  status = pa_get_mod(PA10_ARM_NUMBER, &mode);
  PA10_ERROR_RETURN(status, "qarmGetArmModeCntrl: pa_get_mod", END);
  SDI_F->state.iarm.mode = mode;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10QGetCntrl  -  control codel of CONTROL request Pa10QGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10QGetCntrl(int *report)
{
  long status;
  ANGLE q;

  status = pa_get_agl(PA10_ARM_NUMBER, &q);
  PA10_ERROR_RETURN(status, "qarmGetArmQCntrl: pa_get_agl", END);
  SDI_F->state.iarm.armStatus.angle.s1 = q.s1;
  SDI_F->state.iarm.armStatus.angle.s2 = q.s2;
  SDI_F->state.iarm.armStatus.angle.s3 = q.s3;
  SDI_F->state.iarm.armStatus.angle.e1 = q.e1;
  SDI_F->state.iarm.armStatus.angle.e2 = q.e2;
  SDI_F->state.iarm.armStatus.angle.w1 = q.w1;
  SDI_F->state.iarm.armStatus.angle.w2 = q.w2;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10PosGetCntrl  -  control codel of CONTROL request Pa10PosGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10PosGetCntrl(int *report)
{
  /* better ? return qarmGetArmThCntrl(report); */
  long status;
  VECTOR xyz;

  status = pa_get_xyz(PA10_ARM_NUMBER, xyz);
  PA10_ERROR_RETURN(status, "qarmGetArmThCntrl: pa_get_xyz", END);
  SDI_F->state.iarm.armStatus.noap.vp.x = xyz[0];
  SDI_F->state.iarm.armStatus.noap.vp.y = xyz[1];
  SDI_F->state.iarm.armStatus.noap.vp.z = xyz[2];
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10ThGetCntrl  -  control codel of CONTROL request Pa10ThGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10ThGetCntrl(int *report)
{
  long status;
  MATRIX m;
  VECTOR xyz;

  status = pa_get_noa(PA10_ARM_NUMBER, m);
  PA10_ERROR_RETURN(status, "qarmGetArmThCntrl: pa_get_noa", END);
  SDI_F->state.iarm.armStatus.noap.vx.x = m[0][0];
  SDI_F->state.iarm.armStatus.noap.vx.y = m[1][0];
  SDI_F->state.iarm.armStatus.noap.vx.z = m[2][0];
  SDI_F->state.iarm.armStatus.noap.vy.x = m[0][1];
  SDI_F->state.iarm.armStatus.noap.vy.y = m[1][1];
  SDI_F->state.iarm.armStatus.noap.vy.z = m[2][1];
  SDI_F->state.iarm.armStatus.noap.vz.x = m[0][2];
  SDI_F->state.iarm.armStatus.noap.vz.y = m[1][2];
  SDI_F->state.iarm.armStatus.noap.vz.z = m[2][2];

  status = pa_get_xyz(PA10_ARM_NUMBER, xyz);
  PA10_ERROR_RETURN(status, "qarmGetArmThCntrl: pa_get_xyz", END);
  SDI_F->state.iarm.armStatus.noap.vp.x = xyz[0];
  SDI_F->state.iarm.armStatus.noap.vp.y = xyz[1];
  SDI_F->state.iarm.armStatus.noap.vp.z = xyz[2];
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10QandThGetCntrl  -  control codel of CONTROL request Pa10QandThGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10QandThGetCntrl(int *report)
{
  /* ... add your code here ... */
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10ParametersGetCntrl  -  control codel of CONTROL request Pa10ParametersGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10ParametersGetCntrl(int *report)
{
  long status;
  PARAM* parameters;

  parameters = (PARAM*) &(SDI_F->para.iarm.parameters);
  status = pa_get_prm(PA10_ARM_NUMBER, parameters);
  PA10_ERROR_RETURN(status, "qarmGetArmParametersCntrl: pa_get_prm", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10velocityGetCntrl  -  control codel of CONTROL request Pa10velocityGet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10velocityGetCntrl(int *report)
{
  long status;
  float velocity;

  status = pa_get_inc(PA10_ARM_NUMBER, &velocity);
  PA10_ERROR_RETURN(status, "qarmArmGetVelocityCntrl: pa_get_inc", END);
  SDI_F->iarm.velocity = velocity;
  return OK;
}

/*------------------------------------------------------------------------
 * qarmPa10VelocitySetCntrl  -  control codel of CONTROL request Pa10VelocitySet
 *
 * Description:    
 * Report: OK
 *                 S_qarm_PALIB_ERROR
 *                 S_qarm_OutOfRange
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmPa10VelocitySetCntrl(double *velocity, int *report)
{
  long status;
  float pa10velocity;

  if (*velocity < 0.01 || *velocity > 1) {
    *report = S_qarm_OutOfRange;
    return ERROR;
  }
  /* pa_set_inc : Real-time velocity setting */
  pa10velocity = *velocity;
  status = pa_set_inc(PA10_ARM_NUMBER, pa10velocity);
  PA10_ERROR_RETURN(status, "qarmArmSetVelocityCntrl: pa_set_inc", END);
  return OK;
}

/*------------------------------------------------------------------------
 * qarmTrackQCntrl  -  control codel of EXEC request TrackQ
 *
 * Description:    
 * Report: OK
 *                 S_qarm_POSTER_NOT_FOUND
 *                 S_qarm_BAD_REF
 *                 S_qarm_CMD_STOP_TRACK
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmTrackQCntrl(QARM_STRING_128* posterName, int *report)
{
  if (strlen(posterName->string) > (QARM_MAX_POSTER_NAME - 1) ){
    *report = S_qarm_POSTER_NAME_TOO_LONG;
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmTrackVqCntrl  -  control codel of EXEC request TrackVq
 *
 * Description:    
 * Report: OK
 *                 S_qarm_POSTER_NAME_TOO_LONG
 *                 S_qarm_NOT_IN_MANUAL_MODE
 *                 S_qarm_POSTER_NOT_FOUND
 *                 S_qarm_BAD_REF
 *                 S_qarm_CMD_STOP_TRACK
 *                 S_qarm_COUNTER_MAX
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmTrackVqCntrl(QARM_STRING_128* posterName, int *report)
{
  if (strlen(posterName->string) > (QARM_MAX_POSTER_NAME - 1) ){
    *report = S_qarm_POSTER_NAME_TOO_LONG;
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 * qarmTrackQVqCntrl  -  control codel of EXEC request TrackQVq
 *
 * Description:    
 * Report: OK
 *                 S_qarm_POSTER_NOT_FOUND
 *                 S_qarm_BAD_REF
 *                 S_qarm_CMD_STOP_TRACK
 *
 * Returns:    OK or ERROR
 */

STATUS
qarmTrackQVqCntrl(QARM_STRING_128* posterName, int *report)
{
  if (strlen(posterName->string) > (QARM_MAX_POSTER_NAME - 1) ){
    *report = S_qarm_POSTER_NAME_TOO_LONG;
    return ERROR;
  }
  return OK;
}


