/**
 ** qarmFunctionPA10.c
 **
 ** Functions for codels
 **
 ** Auteur: Daniel SIDOBRE
 ** Date: octobre 2004
 ** 
 **
 **/

#include <portLib.h>
#include "server/qarmHeader.h"

#include "qarmPA10.h"
#include "qarmConst.h"
#include "Proto_gbModeles.h"
#include <stdio.h>
#include <math.h>
#include <sys/time.h>



/* for qarmErrorPrint, qarmErrorStop and qarmErrorFatal we suppose status is 
 * a paLib errorand print the corresponding message. OK and ERR_OK are
 * compatibLE.
 */
STATUS qarmErrorPrint(STATUS status, int line, char* nomFile, char* mess)
{
  fprintf(stderr, 
	  "%s:%d: Pb: (status = %d %s) %s\n", nomFile, line, status,
	  pa10ErrorGetString(status), mess);
  return status;
}

STATUS qarmErrorStop(STATUS status, int line, char* nomFile, char* mess)
{
  qarmEmergencyStop();
  fprintf(stderr, 
	  "%s:%d: PbStop: (status = %d %s) %s\n", nomFile, line, status,
	  pa10ErrorGetString(status), mess);
  return status;
}

STATUS qarmErrorFatal(STATUS status, int line, char* nomFile, char* mess)
{
  qarmFatalErrorStop();
  fprintf(stderr, 
	  "%s:%d: PbFatal: (status = %d %s) %s\n", nomFile, line, status,
	  pa10ErrorGetString(status), mess);
  return status;
}

/*
 *  qarmEmergencyStop : stop the arm, mise en securit et passage en 
 *    mode failed.
 */
STATUS qarmEmergencyStop()
{
  STATUS status;
  
  switch (SDI_F->state.mode) {
  case QARM_STATE_INITIAL: break;
  case QARM_STATE_MANUAL:
  case QARM_STATE_TRACK:
    status = pa_stp_arm(PA10_ARM_NUMBER, WM_NOWAIT);
    if (status != ERR_OK) {
      /* we don't call qarmFatalErrorStop because qarmFatalErrorStop
       *   call qarmEmergencyStop 
       * As we don't necessarily want to stop the module, we prefer
       *  go to the state QARM_STATE_FAILED */
      QARM_ERROR_PRINT(status, "Pb qarmEmergencyStop pa_stp_arm");
      qarmEnd();
      return ERROR;
    }
    SDI_F->state.mode = QARM_STATE_INITIAL;
    break;
  case QARM_STATE_FATAL_ERROR:
  default:
    /* we don't call qarmFatalErrorStop because qarmFatalErrorStop
       call qarmEmergencyStop */
    QARM_ERROR_PRINT(status, "Pb qarmEmergencyStop switch");
    qarmEnd();
    SDI_F->state.mode = QARM_STATE_FATAL_ERROR;
    return ERROR;
  }
  return OK;
}

/*
 * qarmFatalErrorStop : switch to mode QARM_STATE_FATAL_ERROR
 *    it's impossible return from this mode, fatal error due to
 *    pointer or enum problem.
 */
void qarmFatalErrorStop()
{
  qarmEmergencyStop();
  qarmEnd();
  SDI_F->state.mode = QARM_STATE_FATAL_ERROR;
}

/*
 * Stop the arm 
 */
STATUS qarmStopSoft()
{
  STATUS status;

  /* pa_sus_arm : Arm temporarily stop
   * we never use the pa_rsm_arm (Arm temporarily stop release) but
   * some tests shown that new pa10 command can be send to motion
   * control board after pa_sus_arm
   */
  switch (SDI_F->state.mode) {
  case QARM_STATE_INITIAL: break;
  case QARM_STATE_MANUAL:
  case QARM_STATE_TRACK:
    status = pa_sus_arm(PA10_ARM_NUMBER, WM_NOWAIT);
    if (status != ERR_OK) {
      PA10_ERROR_PRINT(status, "Pb pa_sus_arm");
      QARM_ERROR_FATAL(status, "Pb pa_sus_arm");
      return ERROR;
    }
    SDI_F->state.mode = QARM_STATE_MANUAL;
    break;
  case QARM_STATE_FATAL_ERROR:
    break;
  default:
    QARM_ERROR_FATAL(status, "Pb switch");
    return ERROR ;
  }    
  return OK;
}

/*
 *  Stop controller operation : end servo driver communication
 * 
 *  Communication with control board are not closed \ only at the end of
 *  PA library is not terminated                    /   qarm module
 */
STATUS qarmEnd()
{
  STATUS status;

  if (SDI_F->state.simulation == QARM_STATE_REAL)
    status = pa_ext_arm(PA10_ARM_NUMBER);
  else
    status = pa_ext_sim(PA10_ARM_NUMBER);
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_ext_arm/pa_ext_sim");
    SDI_F->state.mode = QARM_STATE_FATAL_ERROR;
    return ERROR;
  }
  SDI_F->state.mode = QARM_STATE_INITIAL;
  return OK;
}

STATUS qarmStartIfNeed()
{
  int status;

  switch(SDI_F->state.mode) {
  case QARM_STATE_INITIAL:
    /* controller operation start, servo driver communication start
     *
     * ATTENTION pa_sta_arm can take a lot of time
     *           pa_sta_sim : a tester
     */
    if (SDI_F->state.simulation == QARM_STATE_REAL)
      status = pa_sta_arm(PA10_ARM_NUMBER);
    else
      status = pa_sta_sim(PA10_ARM_NUMBER);
    if (status != ERR_OK) {
      QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_sta_arm/pa_sta_sim");
      return ERROR;
    }
    break;
  case QARM_STATE_MANUAL:
  case QARM_STATE_TRACK:
  case QARM_STATE_FATAL_ERROR:
    break;
  default:
    QARM_ERROR_FATAL(ERROR, "Pb switch");
    return ERROR;
  }
  return OK;
}

STATUS qarmGetPositionAndSpeed()
{
  int status;

  ARMTARGET armtarget;
  struct timeval s_date;
  static struct timeval s_date0 = { 0, 0};
  double date;
  static double date1 = -1.;
  static double date2 = -1.;
  Gb_th th06;//modif jonathan

  SDI_F->iarm.qold = SDI_F->pos.q;
  SDI_F->iarm.vqold = SDI_F->pos.vq;
  status = pa_get_tar(PA10_ARM_NUMBER, &armtarget);
  if (status != ERR_OK) {
    QARM_ERROR_STOP(status, "qarmGetPositionAndSpeed: pa_get_tar");
    return ERROR;
  }
  SDI_F->pos.q.q1 = armtarget.angle.s1;
  SDI_F->pos.q.q2 = armtarget.angle.s2;
  SDI_F->pos.q.q3 = armtarget.angle.e1;
  SDI_F->pos.q.q4 = armtarget.angle.e2;
  SDI_F->pos.q.q5 = armtarget.angle.w1;
  SDI_F->pos.q.q6 = armtarget.angle.w2;
  
  //modif jonathan  SDI_F->pos.th06.vx.x = armtarget.noap[0][0];
  //modif jonathan  SDI_F->pos.th06.vx.y = armtarget.noap[0][1];
  //modif jonathan  SDI_F->pos.th06.vx.z = armtarget.noap[0][2];
  //modif jonathan  SDI_F->pos.th06.vy.x = armtarget.noap[1][0];
  //modif jonathan  SDI_F->pos.th06.vy.y = armtarget.noap[1][1];
  //modif jonathan  SDI_F->pos.th06.vy.z = armtarget.noap[1][2];
  //modif jonathan  SDI_F->pos.th06.vz.x = armtarget.noap[2][0];
  //modif jonathan  SDI_F->pos.th06.vz.y = armtarget.noap[2][1];
  //modif jonathan  SDI_F->pos.th06.vz.z = armtarget.noap[2][2];
  //modif jonathan  SDI_F->pos.th06.vp.x = armtarget.noap[3][0];
  //modif jonathan  SDI_F->pos.th06.vp.y = armtarget.noap[3][1];
  //modif jonathan  SDI_F->pos.th06.vp.z = armtarget.noap[3][2];

 
  if (gettimeofday(&s_date, 0) != 0) {
    QARM_ERROR_STOP(status, "qarmGetPositionAndSpeed: gettimeofday");
    return ERROR;
  }
  date = 0.000001 * (s_date.tv_usec - s_date0.tv_usec) + 
    (s_date.tv_sec - s_date0.tv_sec);
  SDI_F->pos.date = date;
  if (date1 == -1.) {
    s_date0 = s_date;
    SDI_F->pos.vq.q1 = 0.;
    SDI_F->pos.vq.q2 = 0.;
    SDI_F->pos.vq.q3 = 0.;
    SDI_F->pos.vq.q4 = 0.;
    SDI_F->pos.vq.q5 = 0.;
    SDI_F->pos.vq.q6 = 0.;
  } else {
    SDI_F->pos.vq.q1 =(SDI_F->pos.q.q1 - SDI_F->iarm.qold.q1) / (date - date1);
    SDI_F->pos.vq.q2 =(SDI_F->pos.q.q2 - SDI_F->iarm.qold.q2) / (date - date1);
    SDI_F->pos.vq.q3 =(SDI_F->pos.q.q3 - SDI_F->iarm.qold.q3) / (date - date1);
    SDI_F->pos.vq.q4 =(SDI_F->pos.q.q4 - SDI_F->iarm.qold.q4) / (date - date1);
    SDI_F->pos.vq.q5 =(SDI_F->pos.q.q5 - SDI_F->iarm.qold.q5) / (date - date1);
    SDI_F->pos.vq.q6 =(SDI_F->pos.q.q6 - SDI_F->iarm.qold.q6) / (date - date1);
  }
  if (date2 == -1.) {
    SDI_F->pos.aq.q1 = 0.;
    SDI_F->pos.aq.q2 = 0.;
    SDI_F->pos.aq.q3 = 0.;
    SDI_F->pos.aq.q4 = 0.;
    SDI_F->pos.aq.q5 = 0.;
    SDI_F->pos.aq.q6 = 0.;
  } else {
    SDI_F->pos.aq.q1 = (SDI_F->pos.vq.q1 - SDI_F->iarm.vqold.q1) / (date - date2) * 2.;
    SDI_F->pos.aq.q2 = (SDI_F->pos.vq.q2 - SDI_F->iarm.vqold.q2) / (date - date2) * 2.;
    SDI_F->pos.aq.q3 = (SDI_F->pos.vq.q3 - SDI_F->iarm.vqold.q3) / (date - date2) * 2.;
    SDI_F->pos.aq.q4 = (SDI_F->pos.vq.q4 - SDI_F->iarm.vqold.q4) / (date - date2) * 2.;
    SDI_F->pos.aq.q5 = (SDI_F->pos.vq.q5 - SDI_F->iarm.vqold.q5) / (date - date2) * 2.;
    SDI_F->pos.aq.q6 = (SDI_F->pos.vq.q6 - SDI_F->iarm.vqold.q6) / (date - date2) * 2.;
  }
  date2 = date1;
  date1 = date;
  qarmComputeJac(&th06);//modif jonathan
  SDI_F->pos.th06.vx.x = th06.vx.x;   //modif jonathan
  SDI_F->pos.th06.vx.y = th06.vx.y;   //modif jonathan
  SDI_F->pos.th06.vx.z = th06.vx.z;   //modif jonathan
  SDI_F->pos.th06.vy.x = th06.vy.x;   //modif jonathan
  SDI_F->pos.th06.vy.y = th06.vy.y;   //modif jonathan
  SDI_F->pos.th06.vy.z = th06.vy.z;   //modif jonathan
  SDI_F->pos.th06.vz.x = th06.vz.x;   //modif jonathan
  SDI_F->pos.th06.vz.y = th06.vz.y;   //modif jonathan
  SDI_F->pos.th06.vz.z = th06.vz.z;   //modif jonathan
  SDI_F->pos.th06.vp.x = th06.vp.x;   //modif jonathan
  SDI_F->pos.th06.vp.y = th06.vp.y;   //modif jonathan
  SDI_F->pos.th06.vp.z = th06.vp.z;   //modif jonathan


  return OK;
}

void Gb_jac_x_vq(Gb_jac* jac, Gb_q6* vq, Gb_vitesse* vw)
{
  vw->x  = jac->c1.x  * vq->q1 + jac->c2.x  * vq->q2 + jac->c3.x  * vq->q3 + jac->c4.x  * vq->q4 + jac->c5.x  * vq->q5 + jac->c6.x  * vq->q6;
  vw->y  = jac->c1.y  * vq->q1 + jac->c2.y  * vq->q2 + jac->c3.y  * vq->q3 + jac->c4.y  * vq->q4 + jac->c5.y  * vq->q5 + jac->c6.y  * vq->q6;
  vw->z  = jac->c1.z  * vq->q1 + jac->c2.z  * vq->q2 + jac->c3.z  * vq->q3 + jac->c4.z  * vq->q4 + jac->c5.z  * vq->q5 + jac->c6.z  * vq->q6;
  vw->rx = jac->c1.rx * vq->q1 + jac->c2.rx * vq->q2 + jac->c3.rx * vq->q3 + jac->c4.rx * vq->q4 + jac->c5.rx * vq->q5 + jac->c6.rx * vq->q6;
  vw->ry = jac->c1.ry * vq->q1 + jac->c2.ry * vq->q2 + jac->c3.ry * vq->q3 + jac->c4.ry * vq->q4 + jac->c5.ry * vq->q5 + jac->c6.ry * vq->q6;
  vw->rz = jac->c1.rz * vq->q1 + jac->c2.rz * vq->q2 + jac->c3.rz * vq->q3 + jac->c4.rz * vq->q4 + jac->c5.rz * vq->q5 + jac->c6.rz * vq->q6;
}

STATUS qarmComputeJac(Gb_th *th)  //modif jonathan
{
  Gb_6rParameters pa10;
  Gb_dataMGD data;

  pa10.a2 =  QARM_ARM_A2;
  pa10.r4 =  QARM_ARM_R4;
  pa10.of1 = QARM_ARM_OF1;
  pa10.of2 = QARM_ARM_OF2;
  pa10.of3 = QARM_ARM_OF3;
  pa10.of4 = QARM_ARM_OF4;
  pa10.of5 = QARM_ARM_OF5;
  pa10.of6 = QARM_ARM_OF6;
  pa10.epsilon = QARM_ARM_EPSILON;
  Gb_MGD6rTh(&pa10, &(SDI_F->pos.q), &data, th);  //modif jonathan
  Gb_MDD6r(&pa10, &data, th, &(SDI_F->pos.jac));  //modif jonathan
  Gb_jac_x_vq(&(SDI_F->pos.jac), &(SDI_F->pos.vq), &(SDI_F->pos.vw));
  return OK;
}

#ifdef commentaire_en_developpement
/*
 * / * pour qarm strictement, on n'a besoin que de q * /
 *  ANGLE angle;
 *  status = pa_get_agl(PA10_ARM_NUMBER, &angle);
 *  if (status != ERR_OK) {
 *    QARM_ERROR_STOP(status, "qarmGetPositionAndSpeed: pa_get_agl");
 *    return ERROR;
 *  }
*/
  SDI_F->q.q1 = armtarget.angle.s1;
  SDI_F->q.q2 = armtarget.angle.s2;
  SDI_F->q.q3 = armtarget.angle.e1;
  SDI_F->q.q4 = armtarget.angle.e2;
  SDI_F->q.q5 = armtarget.angle.w1;
  SDI_F->q.q6 = armtarget.angle.w2;

  pa_get_mod 8-113 // STATUS qarmGetArmModeCntrl (int *bilan)
  pa_get_ver 8-115 // STATUS qarmMainTaskInit(int *bilan)
  pa_get_com 8-116
  pa_get_sts 8-117 // STATUS qarmGetArmStatusCntrl (int *bilan)
  pa_get_cnt 8-119 // STATUS qarmGetArmCountCntrl (int *bilan) // compteur
  pa_get_err 8-120 // STATUS qarmGetArmErrorCntrl (int *bilan) // erreur 
  pa_get_xyz 8-122      vecteur xyz
  pa_get_noa 8-123      matrice rotation
  pa_get_ypr 8-124      angles yaw pitch roll
  pa_get_prm 8-125 // STATUS qarmGetArmParametersCntrl (int *bilan) // gains
             max/min divers
  pa_get_tar 8-127 // arm target     angles Q + matrice passage noap + ypr
  pa_get_sav 8-128 acquires each axis servo ON/OFF
  pa_sav_sts 8-129  servo status acquisition
  
  pa_set_ddm 8-131   dead man SW valid/invalid
  pa_get_ddm 8-132

  pa_set_lok  teach lock
  pa_get_lok

  pa_get_max 8-136 nb bras controlables
  pa_get_spt  arm identification
    ;

  pa_set_sim  simulation magnification
  pa_set_inc real-time velocity
  pa_get_sim
  pa_get_inc

  
#endif



/*
  pa_drt_ded    sets arm installation position. 
     (floor mounted / suspending from ceiling)
  pa_get_drt

  pa_chk_cnt
  pa_set_tim time out setting in synchronization processing
  pa_get_tim


  pa_get_sav acquires each axis servo ON/OFF
  pa_sav_sts

  long pa_dio_msk(ARM,long,long,long);  /-* DIO mask setting *-/
  long pa_get_msk(ARM,long,long,long*); /-* DIO mask acquisition *-/

  long pa_set_ddm(ARM,long,long);  /-* Dead man SW valid/invalid *-/
  long pa_get_ddm(ARM,long,long*); /-* Dead man SW valid/invalid status
				      acquisition *-/

  long pa_get_smd(ARM,long*); /-* TEACH MODE acquisition from servo *-/

  pa_get_spt arm identification number retourne 0 ou 1


  long pa_lod_rob(ARM armno,char *file); /-* Robot model file loading *-/
  long pa_lod_tol(ARM armno,char *file); /-* Tool model file loading *-/
  long pa_sav_rob(ARM armno);        /-* model file saving *-/
*/
