/**
 ** qarmMainTaskCodels.c
 **
 ** Codels called by execution task qarmMainTask
 **
 ** Author: 
 ** Date: 
 **
 **/

#ifdef VXWORKS
# include <vxWorks.h>
#else
# include <portLib.h>
#endif
#include <stdio.h>
#include "server/qarmHeader.h"

#include "gbGENOM.h"
#include "qarmPA10.h"


/* mathieu Modif */
/* to read poster */
POSTER_ID poster_kcdg; 
static int qarmTimerMaxSave;


/*------------------------------------------------------------------------
 *
 * qarmMainTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
qarmMainTaskInit(int *report)
{
  long status;
  float f_version;
  double version;

  SDI_F->state.mode = QARM_STATE_INITIAL;
  SDI_F->state.simulation = QARM_STATE_REAL;

  /* PA library initialisation */
  status = pa_ini_sys();
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_ini_sys");
    *report = S_qarm_PALIB_ERROR;
    return ERROR;
  }
  /* open arm, control arm selection */
  status = pa_opn_arm(PA10_ARM_NUMBER);
  if (status < ERR_OK) { /* return 8 ??? */
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_opn_arm");
    *report = S_qarm_PALIB_ERROR;
    return ERROR;
  }
  status = pa_rst_ctl(PA10_ARM_NUMBER);
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_rst_ctl");
    *report = S_qarm_PALIB_ERROR;
    return ERROR;
  }
  status = pa_get_ver(PA10_ARM_NUMBER, &f_version);
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_get_ver");
  } else {
    version = f_version;
    printf("palib version is %g\n", version);
  }
  SDI_F->vqConsigne.q1 = 0.0;
  SDI_F->vqConsigne.q2 = 0.0;
  SDI_F->vqConsigne.q3 = 0.0;
  SDI_F->vqConsigne.q4 = 0.0;
  SDI_F->vqConsigne.q5 = 0.0;
  SDI_F->vqConsigne.q6 = 0.0;
  SDI_F->iarm.timeOutPCICard = 20; /* en 10 ms  soit 20 correspond a 200 ms
				    Utilisé par le tracking*/
  return OK;
}

/*------------------------------------------------------------------------
 *
 * qarmMainTaskEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
qarmMainTaskEnd(void)
{
  long status;

  qarmFatalErrorStop();
  /* close arm, control arm separation */
  status = pa_cls_arm(PA10_ARM_NUMBER);
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_cls_arm");
    return ERROR;
  }  
  /* PA library termination */
  status = pa_ter_sys();
  if (status != ERR_OK) {
    QARM_ERROR_PRINT(status, "qarmMainTaskInit: pa_ter_sys");
    return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 *
 * qarmMainTaskPerm  --  codel of permanent activity
 *
 * Description: 
 * 
 * Reports:      OK
 *              S_qarm_PALIB_ERROR
 * 
 * Returns:    OK or ERROR
 */

STATUS
qarmMainTaskPerm(int *report)
{
  static float speed[7];
  long status;

  if (SDI_F->state.mode != QARM_STATE_FATAL_ERROR) {
    if (qarmGetPositionAndSpeed() != OK) {
      QARM_ERROR_FATAL(0, "A faire ???"); /* ???*/
    }
  }
  switch (SDI_F->state.mode) {
  case QARM_STATE_INITIAL:
  case QARM_STATE_MANUAL:
    /* do nothing : position control by pci-motion-control-card */
    break;
  case QARM_STATE_TRACK:
    speed[0] = SDI_F->vqConsigne.q1;
    speed[1] = SDI_F->vqConsigne.q2;
    speed[2] = 0.0;
    speed[3] = SDI_F->vqConsigne.q3;
    speed[4] = SDI_F->vqConsigne.q4;
    speed[5] = SDI_F->vqConsigne.q5;
    speed[6] = SDI_F->vqConsigne.q6;
    /*  pa_set_tim time out setting  il faut le re'gler quelque part
	default 1000 ms 
	pa_chk_cnt synchronisation in weight compensasion, must be issued 
	  once in 1000 ms 
    */
    status = pa_odr_vel(PA10_ARM_NUMBER, speed);
    if (status != ERR_OK) {
      QARM_ERROR_PRINT(status, "qarmMainTaskPerm: pa_odr_vel");
      return END;
    }
    break;
  case QARM_STATE_FATAL_ERROR:
  default:
    return QARM_ERROR_FATAL(ERROR, "Pb switch");
  }
  return OK;
}
/*------------------------------------------------------------------------
 * Warning_col
 *
 * Description: 
 *
 * Reports:      OK
 *              S_qarm_POSTER_NOT_FOUND
 */

/* qarmWarning_colStart  -  codel START of Warning_col
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmWarning_colStart(QARM_STRING_128 *(collisionPoster), int *report)
{
  /* mathieu Modif */
  // find the poster 
  if (posterFind(collisionPoster->string, &poster_kcdg) == ERROR) {
    /*    printf("Cannot find %s\n", collisionPoster->string); */
    QARM_ERROR_PRINT(ERROR, collisionPoster->string);
    *report =  S_qarm_POSTER_NOT_FOUND ; 
    return ETHER;
  }
  /*  printf("poster found - OK\n") ; */
  qarmTimerMaxSave = -1 ;
  return EXEC;
}

/* qarmWarning_colExec  -  codel EXEC of Warning_col
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmWarning_colExec(QARM_STRING_128 *(posterName), int *report)
{
  QARM_COLLISION_CMD_STR col_flag ;

  if (poster_kcdg != NULL) {
    qarmQARM_COLLISION_CMD_STRPosterRead(poster_kcdg, &col_flag);
  } else {
    *report = S_qarm_POSTER_NOT_FOUND ;
    return ETHER ;
  }
  //first time
  if (qarmTimerMaxSave == -1) qarmTimerMaxSave = col_flag.timer_max ;
  //others
  if ( (col_flag.flag == 1) || (col_flag.timer >= qarmTimerMaxSave)) {
    if(col_flag.timer >= qarmTimerMaxSave) {
      printf("MISS %d poster \n",col_flag.timer-qarmTimerMaxSave+1);
      QARM_ERROR_PRINT(ERROR, "Pb timer");
    }
    qarmStopSoft();
    QARM_ERROR_PRINT(0, "QARM_STOP\n");
    return ETHER ;
  }
  else {
    qarmTimerMaxSave = col_flag.timer_max ;
    return EXEC;
  }
}

/*------------------------------------------------------------------------
 * GotoQWait
 *
 * Description: 
 *
 * Reports:      OK
 *              S_qarm_Q_LIMITED
 *              S_qarm_NOT_IN_MANUAL_MODE
 */
static long QWaitControlCounter;
/* qarmGotoQWaitStart  -  codel START of GotoQWait
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmGotoQWaitStart(Gb_q6 *qConsigne, int *report)
{
  long status;
  ANGLE angle;

  if (SDI_F->state.mode != QARM_STATE_MANUAL) {
    QARM_ERROR_PRINT(ERROR, "qarmGotoQCntrl");
    *report = S_qarm_NOT_IN_MANUAL_MODE;
    return END;
  }
  angle.s1 = qConsigne->q1;
  angle.s2 = qConsigne->q2;
  angle.s3 = 0.;
  angle.e1 = qConsigne->q3;
  angle.e2 = qConsigne->q4;
  angle.w1 = qConsigne->q5;
  angle.w2 = qConsigne->q6;
  status = pa_get_cnt(PA10_ARM_NUMBER, &QWaitControlCounter);
  PA10_ERROR_RETURN(status, "pa_get_cnt", END);
  status = pa_exe_axs(PA10_ARM_NUMBER, ALLAXIS, &angle, WM_NOWAIT);
  PA10_ERROR_RETURN(status, "pa_exe_axs", END);
  return EXEC;
}

/* qarmGotoQWaitExec  -  codel EXEC of GotoQWait
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmGotoQWaitExec(Gb_q6 *qConsigne, int *report)
{
  long status;
  long counter;

  status = pa_get_cnt(PA10_ARM_NUMBER, &counter);
  PA10_ERROR_RETURN(status, "pa_get_cnt", END);
  if (QWaitControlCounter != counter) {
    return END;
  } else {
    return EXEC;
  }
}

/*------------------------------------------------------------------------
 * QRelativeSpeed
 *
 * Description: 
 *
 * Reports:      OK
 *              S_qarm_NOT_IN_MANUAL_MODE
 *              S_qarm_Q_LIMITED
 *              S_qarm_NOT_WRITTEN
 */

/* qarmQRelativeSpeedStart  -  codel START of QRelativeSpeed
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmQRelativeSpeedStart(QARM_QRELATIVE_STR *delta, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

/* qarmQRelativeSpeedExec  -  codel EXEC of QRelativeSpeed
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmQRelativeSpeedExec(QARM_QRELATIVE_STR *delta, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

/* qarmQRelativeSpeedEnd  -  codel END of QRelativeSpeed
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmQRelativeSpeedEnd(QARM_QRELATIVE_STR *delta, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}

/* qarmQRelativeSpeedInter  -  codel INTER of QRelativeSpeed
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
qarmQRelativeSpeedInter(QARM_QRELATIVE_STR *delta, int *report)
{
  /* ... add your code here ... */
  return ETHER;
}


