#ifndef QARMSTRUCT_H
#define QARMSTRUCT_H
/*
 *  qarmStruct.h

 * Modified le 7 juillet 2004
 * Par Ignacio HERRERA AGUILAR
 * Commande en vitesse de la position et vitesse angular de chaque servo
 * 

 *
 *  Daniel SIDOBRE
 *
 * Copyright (c) 2004 LAAS/CNRS 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Here is included qarmStructPa10.h. This file contain specific 
 *    definitions for the type of robot pa10. For exemple specific
 *    data structures for adjustment...
 */
#include "qarmStructPa10.h"

#include "gbGENOM.h"

#define QARM_ERROR_PRINT(S, M) qarmErrorPrint(S, __LINE__, __FILE__, M)
#define QARM_ERROR_STOP(S, M)  qarmErrorStop( S, __LINE__, __FILE__, M)
#define QARM_ERROR_FATAL(S, M) qarmErrorFatal(S, __LINE__, __FILE__, M)

/*constantes */

typedef enum QARM_ON_OFF_ENUM {
  QARM_OFF = 0,
  QARM_ON = 1
} QARM_ON_OFF_ENUM;

typedef enum QARM_COLLISION_STATE_ENUM {
  QARM_COLLISION_FREE = 0,
  QARM_COLLISION_COLLISION
} QARM_COLLISION_STATE_ENUM;

typedef struct QARM_COLLISION_STATE_STR {
  QARM_ON_OFF_ENUM mode;
  QARM_COLLISION_STATE_ENUM state;
} QARM_COLLISION_STATE_STR;

typedef enum QARM_STATE_ENUM {
  QARM_STATE_INITIAL,     /* state at initialisation */
  QARM_STATE_MANUAL,      /* debug mode to acces some functions before
			     POM is done */ 
  QARM_STATE_TRACK,       /* mode tracking (NORMAL mode with tracking)
			  un seul mode car genom fait le travail  partir 
			  de plusieurs requettes d'execution */
  QARM_STATE_FATAL_ERROR  /* fatal error --> end of task */
} QARM_STATE_ENUM;
/*
 case QARM_STATE_INITIAL:
 case QARM_STATE_MANUAL:
 case QARM_STATE_TRACK:
 case QARM_STATE_FATAL_ERROR:
 default:
*/

/*
 *  data for poster to export position, velocity, jacobian ...
 */
typedef struct QARM_POSITION_STR {
  Gb_q6 q;
  Gb_th th06;
  Gb_q6 vq;
  Gb_q6 aq;
  Gb_jac jac;
  Gb_vitesse vw;
  double date;
} QARM_POSITION_STR;

typedef enum QARM_SIMULATION_ENUM {
  QARM_STATE_REAL,      /* Normal use of the arm */
  QARM_STATE_SIMULATION /* Simulation at PA10 Motion Control Board level */
} QARM_SIMULATION_ENUM;

/*
 * mode defines two normal modes ( QARM_STATE_NORMAL and
 *   QARM_STATE_TRACK) and initialisations mode. QARM_STATE_MANUAL is for
 *   use of Mitsubishi motion control board functions.
 *
 */

typedef struct QARM_STATE_STR {
  QARM_STATE_ENUM mode;
  QARM_SIMULATION_ENUM simulation;
  QARM_ON_OFF_ENUM speedLimitMode;
  int inutileAlignement;

  QARM_COLLISION_STATE_STR collision;
  QARM_INSTANCE_STATE_STR iarm;
} QARM_STATE_STR;

typedef struct QARM_INSTANCE_PARAMETERS_STR {
  PA10_PARAMETERS_STR parameters;
} QARM_INSTANCE_PARAMETERS_STR;

typedef struct QARM_PARAMETERS_STR {
  double bientot;
  QARM_INSTANCE_PARAMETERS_STR iarm;
} QARM_PARAMETERS_STR;

typedef struct QARM_QRELATIVE_STR {
  Gb_q6 vq;
  Gb_q6 deltaQ;
} QARM_QRELATIVE_STR;

typedef enum QARM_TRACK_MODE_ENUM {
  QARM_TRACK_MODE_NO,
  QARM_TRACK_MODE_Q,
  QARM_TRACK_MODE_VQ,
  QARM_TRACK_MODE_QVQ
} QARM_TRACK_MODE_ENUM;

/* maximal size of a poster name */
#define QARM_MAX_POSTER_NAME 128
typedef struct QARM_STRING_128 {
  char string[QARM_MAX_POSTER_NAME];
} QARM_STRING_128;


typedef struct QARM_TRACK_STR {
  QARM_STRING_128 posterName;
  QARM_TRACK_MODE_ENUM trackMode;        
  int posterId;    
} QARM_TRACK_STR;

typedef struct QARM_COLLISION_CMD_STR {
  
  int flag ;
  int unused0 ;

  int timer ; 
  int unused1 ;

  int timer_max ;
  int unused2 ;

  int speed_max ;
  int unused3 ;

} QARM_COLLISION_CMD_STR ;

typedef struct QARM_VELOCYTY_TRACK_STR {
  int counter;      /* counter, must be updated */ 
  int counterMax;   /* maximum number of tic before new counter value
                     *   or the call of qarmEmergencyStop() */
  Gb_q6 qVelocityC; /* velocities set point */
} QARM_VELOCYTY_TRACK_STR;

#endif
