#ifndef QARM_STRUCT_PA10_H
#define QARM_STRUCT_PA10_H

/*
 *  qarmStructPa10.h
 *
 *  Daniel SIDOBRE
 *
 * Copyright (c) 2005 LAAS/CNRS 
 */

/*
 *  This file contain specific definition for the instance of module 
 *     qarm for Mitsubishi PA10-6C arm.
 */

#include "gbGENOM.h"

typedef struct PA10_ANGLE_STR {
  double s1;
  double s2;
  double s3; /* not use for 6-axis arm */
  double e1;
  double e2;
  double w1;
  double w2;
} PA10_ANGLE_STR;

typedef struct PA10_STATUS_STR {
  long   max;             /* Board controllable arm numbers   1or2 */
  long   arm;             /* Arm identification number 0or1 */
  long   axis;            /* Arm axis numbers */
  long   typ;             /* Arm type */
  long   drv;             /* Servo driver classification */
  long   dio;             /* Extension DIO board      exist  /  not exist */
  long   remote;          /* operation mode (valid / invalid) */
  long   count;           /* Control Counter value */
  long   error;	          /* Error code */
  PA10_ANGLE_STR  angle;  /* Current axis value */
  Gb_th noap;             /* Current tip orientation matrix */
  double ypr[3];          /* Current orientation */ /*- ypr: Yaw Pitch Roll -*/
} PA10_STATUS_STR;

typedef struct PA10_PARAMETERS_STR {
  float	rezl;        /* Resolver resolution */
  float	pul[7];      /* Position limiter (+) */
  float	pdl[7];      /* Position limiter (-) */
  float	vel[7 + 2];  /* Velocity limiter */
  float	dev[7 + 2];  /* Default Velocity */
  float	lim[7 + 2];  /* 				*/
  float	ceh[7 + 2];  /* 				*/
  float	cem[7 + 2];  /* 				*/
  float	cel[7 + 2];  /* 				*/
  float	pg1[7];      /* Position control gain 1 */
  float	pg2[7];      /* Position control gain 2 */
  float	vg1[7];      /* Velocity control gain */
  float	tg1[7];      /* Force control gain */
  float	pcm[7];      /* position control selection matrix */
  float	fcm[7];      /* Force control selection matrix */
  float	arl[7];      /* Arm length */
  float	arg[7];      /* Axis gravity center position */
  float	arw[7];      /* Axis weight */
  float	hom[7];      /* Home position recovery target value */
  float	saf[7];      /* Safety position recovery target value */
  float	esc[7];      /* Escape position recovery target value */
  float	tol[7];      /* Tool parameter */
  float	fvl[7];      /* 				*/
  long	dmy[7];      /* */
  long	spa[7];      /* Spare */
  int inutileAlignement;
} PA10_PARAMETERS_STR;
  
typedef struct PA10_QANDTH_STR {
  PA10_ANGLE_STR  angle;  /* Current axis value */
  Gb_th noap;             /* Current tip orientation matrix */
  double ypr[3];          /* Current orientation */ /*- ypr: Yaw Pitch Roll -*/
} PA10_QANDTH_STR;

typedef enum PA10_MODE_ENUM {
  PA_NOT_AVAILABLE_1                  =  1,
  PA_NOT_AVAILABLE_2                  =  2,
  PA_BRAKE_STOP                       =  3,
  PA_NOT_AVAILABLE_4                  =  4,
  PA_NOT_AVAILABLE_5                  =  5,
  PA_NOT_AVAILABLE_6                  =  6,
  PA_NOT_AVAILABLE_7                  =  7,
  PA_EACH_AXIS_ANGGLE_CONTROL         =  8,
  PA_EACH_AXIS_VELOCITY_CONTROL       =  9,
  PA_DIRECT_SERVO_LOCK                = 10,
  PA_SIMPLIFIED_WEIGHT_COMPENSATION   = 11,
  PA_WEIGHT_CONPENSATION              = 12,
  PA_RMRC_CONTROL                     = 13,
  PA_RMRC_REDUNDANT_AXIS_CONTROL      = 14,
  PA_EACH_AXIS_CONTROL_SERVO_LOCK     = 15,
  PA_NOT_AVAILABLE_16                 = 16,
  PA_EACH_AXIS_ANGLE_CORRECTION       = 17,
  PA_CIRCLE_INTERPOLATION_PLAYBACK    = 18,
  PA_LINEAR_INTERPOLATION_PLAYBACK    = 19,
  PA_ARC_INTERPOLATION_PLAYBACK       = 20,
  PA_RMRC_CONTROL_SERVO_LOCK          = 21,
  PA_PLAYBACK_START_WAITING_EACH_AXIS = 22,
  PA_EACH_AXIS_CONTROL_SERVO_LOCK_P   = 23,
  PA_RMRC_CONTROL_SERVO_LOCK_P        = 24,
  PA_PLAYBACK_START_WAITING_RMRC      = 25,
  PA_PLAYBACK_TIP_POSITION_SHIFTING   = 26,
  PA_REDUNDANT_AXIS_MOVABLE           = 27,
  PA_RMRC_REAL_TIME                   = 28,
  PA_PLAYBACK_AXIS_INTERPOLING_ANGLE_CORRECTION = 29,
  PA_INTERIM_STATUS_SHIFTING_TO_POINT_AFTER_COORDINATE /* CONVERSION */ = 30,
  PA_REDUNDANT_AXIS_MOVABLE_S3        = 31,
  PA_EACH_AXIS_REAL_TIME_CONTROL_MODE = 32,
  PA_MOTION_BETWEEN_TEACH_DATA_RMRC   = 33,
  PA_MOTION_BETWEEN_TEACH_DATA_EACH   = 34
} PA10_MODE_ENUM;


typedef struct QARM_INSTANCE_STATE_STR {
  PA10_STATUS_STR armStatus;
  PA10_QANDTH_STR QandTh;
  PA10_MODE_ENUM mode;
} QARM_INSTANCE_STATE_STR;


typedef struct QARM_INSTANCE_STR {
  double velocity;
  int timeOutPCICard;
  int inutileAlignement;
  Gb_q6 qold;
  Gb_q6 vqold;
} QARM_INSTANCE_STR;

#endif
